package com.docus.server.report.listener;

import cn.hutool.core.net.Ipv4Util;
import cn.hutool.http.HttpRequest;
import cn.hutool.http.HttpResponse;
import cn.hutool.http.HttpUtil;
import cn.hutool.json.JSONObject;
import com.alibaba.fastjson.JSON;
import com.docus.core.util.Func;
import com.docus.server.report.config.ApplicationBusinessConfig;
import com.docus.server.report.dto.ReportDownDto;
import com.docus.server.report.dto.ReportDownPatientDto;
import com.docus.server.report.dto.ReportDownScanFileDto;
import com.docus.server.report.dto.ReportDto;
import com.docus.server.report.event.ReportDownEvent;
import com.docus.server.report.mapper.AfReportRecordMapper;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.event.EventListener;
import org.springframework.scheduling.annotation.Async;
import org.springframework.stereotype.Component;

import javax.annotation.Resource;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;

/**
 * 上报下载监听
 * @author wyb
 */
@Component
@Slf4j
public class ReportDownListener {
    @Resource
    private ApplicationBusinessConfig applicationBusinessConfig;

    @Resource
    private AfReportRecordMapper afReportRecordMapper;

    @EventListener
    @Async("threadPoolExecutor")
    public void reportDown(ReportDownEvent reportDownEvent){
        ReportDto reportDto = reportDownEvent.getReportDto();
        // 组织基础信息数据
        ReportDownPatientDto reportDownPatientDto = new ReportDownPatientDto(reportDto);

        // 组织文件数据
        List<ReportDownScanFileDto> reportDownScanFileDtos = new ArrayList<>(5);
        ReportDownScanFileDto reportDownScanFileDto = new ReportDownScanFileDto(reportDto);
        reportDownScanFileDtos.add(reportDownScanFileDto);

        // 组织下载数据，基础信息和文件数据
        ReportDownDto reportDownDto = new ReportDownDto();
        reportDownDto.setAssortid(reportDto.getAssortId());
        reportDownDto.setCollectorid(reportDto.getSysFlag());
        reportDownDto.setScanusercode("admin");
        reportDownDto.setScanusername("admin");
        reportDownDto.setPatient(reportDownPatientDto);
        reportDownDto.setScanfiles(reportDownScanFileDtos);

        // 下面是调用下载服务
        String requestParam = JSON.toJSONString(reportDownDto);
        try {
            HttpRequest post = HttpUtil.createPost(applicationBusinessConfig.getDownUrl());
            post.timeout(5*1000);
            post.header("Content-Type","application/json; charset=utf-8");
            post.body(requestParam);
            HttpResponse response = post.execute();
            String respBody = response.body();
            log.info("调用下载服务成功,响应参数：{}",respBody);
            afReportRecordMapper.updateStateByTaskId(reportDto.getTaskId());
        }catch (Exception e){
            log.error("调用下载服务失败：地址为："+applicationBusinessConfig.getDownUrl()+" 参数为 ： "+requestParam,e);
        }
    }
}
